% Sub-Matlab program for :
% Gapen, Michael T. and Thomas F. Cosimano, 
% "Sovling Ramsey Problems with Nonlinear Projection Methods," 
% Studies in Nonlinear Dynamics and Econometrics, Vol. 9, Spring
% 2005.
%
% x=chebnode(n);
%
% Purpose: Create n Chebyshev nodes
%
% This program creates n Chebyshev nodes across the interval [-1,1].  The
% higher the n, the more nodes and the grid will be finer. The nodes are the
% zeros of the Chebysheve polynomials
%
% Source: Christian Haefke and Wouter den Haan, UCSD
% ======================================================================
function x=chebnode(n);

r	= max(1,n);
n	= (1:n)';
x	= cos((pi*(n-0.5))/r);

% ======================================================================
% MAX    Largest component.
%    For vectors, MAX(X) is the largest element in X. For matrices,
%    MAX(X) is a row vector containing the maximum element from each
%    column. For N-D arrays, MAX(X) operates along the first
%    non-singleton dimension.
%
% COS    Cosine.
%    COS(X) is the cosine of the elements of X. 
%
% The classical Chebyshev polynomials arose as a solution to the
% problem of finding a function with the least deviation from zero. 

