% Sub-Matlab program for :
% Gapen, Michael T. and Thomas F. Cosimano, 
% "Sovling Ramsey Problems with Nonlinear Projection Methods," 
% Studies in Nonlinear Dynamics and Econometrics, Vol. 9, Spring
% 2005.
%
% x=chebpol(ord,p);
%
% Purpose: Create an ord-th order Chebyshev Polynomial based on the vector p
%
% Source: Christian Haefke and Wouter den Haan, UCSD
%
% ===========================================================
function x=chebpol(ord,p);

[r c]	= size(p);
 	x	= ones(r,ord+1);
    x(:,2)	= p;
	if ord > 2;
	   for i	= 3:ord+1;
		x(:,i)	= 2.*p.*x(:,i-1)-x(:,i-2);
           end;
        end;
        
% ============================================================
%	  [M,N] = SIZE(X) returns the number of rows and columns in
%    separate output variables. [M1,M2,M3,...,MN] = SIZE(X)
%    returns the length of the first N dimensions of X.
%
% The Chebyshev polynomials are of the recursive scheme:
% 
% T(n+1)(x) = 2x*T(n)(x) - T(n-1)(x)
%
% where n relates to the order and the recursion scheme evaluates the polynomials at x.
% In the function x=chebpol(ord,p) statement, the p refers to the vector of or point of 
% x and the loop calculates the polynomial based on the order specified in ord.  