% Sub-Matlab program for :
% Gapen, Michael T. and Thomas F. Cosimano, 
% "Sovling Ramsey Problems with Nonlinear Projection Methods," 
% Studies in Nonlinear Dynamics and Econometrics, Vol. 9, Spring
% 2005.
%
function [x,rc] = csolvesimsnew(FUN,x,points,gradfun,crit,itmax,varargin)

global alpha beta a rhotheta rhog sigmatheta sigmag thetamax thetamin gmax gmin
global thetaord gord tnod gnod G0 theta0 Y maxiter coefftol gamma
global C1 C2 M mu G_Y C1_Y H g0 Kt D_Y X

%function [x,rc] = csolvesims(FUN,x,gradfun,crit,itmax,varargin)
% FUN should be written so that any parametric arguments are packed in to xp,
% and so that if presented with a matrix x, it produces a return value of
% same dimension of x.  The number of rows in x and FUN(x) are always the
% same.  The number of columns is the number of different input arguments
% at which FUN is to be evaluated.
%
% gradfun:  string naming the function called to evaluate the gradient matrix.  If this
%           is null (i.e. just "[]"), a numerical gradient is used instead.
% crit:     if the sum of absolute values (or norm) that FUN returns is less than this,
%           the equation is solved.
% itmax:    the solver stops when this number of iterations is reached, with rc=4
% varargin: in this position the user can place any number of additional arguments, all
%           of which are passed on to FUN and gradfun (when it is non-empty) as a list of 
%           arguments following x.
% rc:       0 means normal solution, 1 and 3 mean no solution despite extremely fine adjustments
%           in step length (very likely a numerical problem, or a discontinuity). 4 means itmax
%           termination.
%
format long;
%---------- delta --------------------
% differencing interval for numerical gradient
% delta = 1e-6; % Original value
delta=1e-6;
%-------------------------------------
%------------ alpha ------------------
% tolerance on rate of descent
alpha1=1e-3;
%-------------------------------------
%------------ verbose ----------------
% if this is set to zero, all screen output is suppressed
verbose=1;
%-------------------------------------
%-------------itctprnt----------------
% how often screen output is shown. To print every iteration,
% set itctprnt=0.
itctprnt = 1;
%-------------------------------------
%------------ analyticg --------------
analyticg=1-isempty(gradfun); 
% if the grad argument is [], numerical derivatives are used.
% isempty(x) returns 1 if x is an empty array and 0 otherwise.
%-------------------------------------
nv=length(x);
tvec=delta*eye(nv);
done=0;
if isempty(varargin)
   f0=feval(FUN,x,points);
else
   f0=feval(FUN,x,points,varargin{:});
end   
%af0=sum(abs(f0));
af0=norm(f0);
af00=af0;
itct=0;
level=0;
while ~done 
    tic;    
   if itct>3 & af00-af0<crit*max(1,af0) & rem(itct,2)==1 
       % REM    Remainder after division.
       % REM(x,y) is x - y.*fix(x./y) if y ~= 0.  By convention, REM(x,0) is NaN.
       % The input x and y must be real arrays of the same size, or real scalars.
      randomize=1;
   else
       if ~analyticg
          % ~ is the LOGICAL NOT operator.  ~A will return a 1 where the
          % elements of A are zero and zeros when elements of A are non-zero.
          % If gradfun is empty above, analyticg=0 prior to the ~analyticg command.
          % Then it proceeds inside the loop below to calculate the gradient matrix.
         if isempty(varargin)
             %Old process for gradient matrix below
                j         = 0;
                for    e  = eye(nv),
                        % When the 'for' statement is used with an expression,
                        % the columns of the expression are stored one at a time in
                        % the variable and then the following statements, up to the
                        % END, are executed.  Thus, the next line takes the first column
                        % of the identity matrix. Then the second, third...until n is reached.
                        xd     = x + delta*e;
                        [Fd]   = feval(FUN,xd,points);
                        j      = j + 1;
                        % The matrix below stores the differenced jacobian.  The matrix will be
                        % updated below.
                        grad(:,j) = Fd - f0;
                    end;
                % Uncover the commands below to check the performance of the gradient matrix.
                %grad1=grad;
                rcond1=rcond(grad);
                %inv1=inv(grad)
                %
            %grad = (feval(FUN,x*ones(1,nv)+tvec,points)-f0*ones(1,nv))/delta;
         else
            grad = (feval(FUN,x*ones(1,nv)+tvec,points,varargin{:})-f0*ones(1,nv))/delta;
         end
      else % use analytic gradient
         grad=feval(gradfun,x,points,varargin{:});
      end
      if isreal(grad)
         % isreal(x) returns 1 if x does not have an imaginary part and 0 otherwise  
         if rcond(grad)<1e-8;
             grad=grad+tvec;
         end
         dx0=-grad\f0;
         randomize=0;
      else
         %if(verbose) & level == itctprnt,disp('gradient imaginary'),end
         if(verbose) ,disp('gradient imaginary'),end
         randomize=1;
      end
   end
   if randomize
       %if(verbose) & level == itctprnt,fprintf(1,'\n Random Search'),end
       if(verbose) ,fprintf(1,'\n Random Search'),end
       dx0=norm(x)./randn(size(x));
   end
   lambda=1;
   lambdamin=1;
   fmin=f0;
   xmin=x;
   afmin=af0;
   dxSize=norm(dx0); %norm chooses the largest value in the vector.
   %here - the smaller the factor, the quicker af scales down? Original program =0.6
   % If change factor here, must change below, too.
   factor=.6;
   shrink=1;
   subDone=0;
   while ~subDone
      %lambdadx=lambda;
      dx=lambda*dx0;
      f=feval(FUN,x+dx,points,varargin{:});
      af=norm(f);
      if (af<afmin) 
         afmin=af;
         fmin=f;
         lambdamin=lambda;
         xmin=x+dx;
      end
      if ((lambda >0) & (af0-af < alpha1*lambda*af0)) | ( (lambda<0) & (af0-af < 0) )
          % The "|" symbol means "or".
         if ~shrink
            factor=factor^.6;
            shrink=1;
         end
         if (abs(lambda*(1-factor))*dxSize > .1*delta);
            lambda = factor*lambda;
            %here - if change factor above, must change below, too. One more below.
         elseif (lambda > 0) & (factor==.6)  %i.e., we've only been shrinking
            lambda=-.3;
         else %
            subDone=1;
            if lambda > 0
                %here also.
               if factor==.6 
                  rc = 2;
               else
                  rc = 1;
               end
            else
               rc=3;
            end
         end
      elseif (lambda >0) & (af-af0 > (1-alpha1)*lambda*af0) 
         if shrink
            factor=factor^.6;
            shrink=0;
         end
         lambda=lambda/factor;
      else
          % good value found
          subDone=1;
          rc=0;
      end
   end % while ~subDone
   timer=toc;
   itct=itct+1;
   level=level+1;
   if(verbose) & level == itctprnt
       %A=grad1
       %invA=inv(grad1)
       %dx=-grad1\f0
      lamx=xmin(1:(thetaord+1)*(gord+1));
      phix=xmin(((thetaord+1)*(gord+1)+1):(2*(thetaord+1)*(gord+1)));
      psix=xmin((2*(thetaord+1)*(gord+1)+1):(3*(thetaord+1)*(gord+1)));
      omegax=xmin((3*(thetaord+1)*(gord+1)+1):(4*(thetaord+1)*(gord+1)));
      l=lambdafun(1.0,1.0,lamx);
      m=-log(mfun(1.0,1.0,psix));   % converts exp-(mu) to mu.
      %m=1/(mfun(1.0,1.0,psix));    % the gross growth rate of money.
      t=taufun(1.0,1.0,omegax);
      h=hfun(1.0,1.0,phix);
      ERL=rlquad(phix,psix);
      out=exp(theta0)*Kt*h^alpha;
      mm=mfun(1.0,1.0,psix);
      xx=X;
      cc1=((Y-X-exp(g0)*G0)*beta*(a/(1-a))*mm)/(1+beta*(a/(1-a))*mm);
      cc2=(Y-X-exp(g0)*G0)/(1+beta*(a/(1-a))*mm);
      fprintf(1,'\nit %d, af %g, rc %g, timer %.1f, l %.10f, m %.10f, t %.10f, h %.10f'...
          ,itct,afmin,rc,timer,l,m,t,h)
      fprintf(1,'\ncc1 %.10f, cc2 %.10f, rl %.10f',cc1,cc2,ERL)  
      
      level=0;
  end
   x=xmin;
   f0=fmin;
   af00=af0;
   af0=afmin;
   if itct >= itmax
      done=1;
      rc=4;
   elseif af0<crit;
      done=1;
      rc=0;
   end
end

fmin
%fprintf(1,'\n %10g',x);







